/**
 * =========================================
 * LibXML : a free Java layouting library
 * =========================================
 *
 * Project Info:  http://reporting.pentaho.org/libxml/
 *
 * (C) Copyright 2006-2008, by Object Refinery Ltd, Pentaho Corporation and Contributors.
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 *
 *
 * ------------
 * PropertiesReadHandler.java
 * ------------
 */
package org.pentaho.reporting.libraries.xmlns.parser;

import java.util.ArrayList;
import java.util.Properties;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * A read handler that creates property-structures (name-value-pairs) and returns the properties as java.util.Properties
 * collection.
 *
 * @author Thomas Morgner
 */
public class PropertiesReadHandler extends AbstractXmlReadHandler
{
  private ArrayList propertyHandlers;
  private String propertyTagName;
  private Properties result;

  /**
   * Creates a properties read-handler using "property" as child-tagname.
   */
  public PropertiesReadHandler()
  {
    this("property");
  }


  /**
   * Creates a properties read-handler using the specified propertyTagName as child-tagname.
   *
   * @param propertyTagName the tag name for the child elements that define the properties.
   */
  public PropertiesReadHandler(final String propertyTagName)
  {
    if (propertyTagName == null)
    {
      throw new NullPointerException();
    }
    this.propertyHandlers = new ArrayList();
    this.propertyTagName = propertyTagName;
  }

  /**
   * Returns the handler for a child element.
   *
   * @param tagName the tag name.
   * @param atts    the attributes.
   * @return the handler or null, if the tagname is invalid.
   * @throws SAXException if there is a parsing error.
   */
  protected XmlReadHandler getHandlerForChild(final String uri,
                                              final String tagName,
                                              final Attributes atts)
      throws SAXException
  {
    if (isSameNamespace(uri) == false)
    {
      return null;
    }
    if (tagName.equals(propertyTagName))
    {
      final PropertyReadHandler prh = new PropertyReadHandler();
      propertyHandlers.add(prh);
      return prh;
    }
    return null;
  }

  /**
   * Done parsing.
   *
   * @throws SAXException if there is a parsing error.
   */
  protected void doneParsing() throws SAXException
  {
    result = new Properties();
    for (int i = 0; i < propertyHandlers.size(); i++)
    {
      final PropertyReadHandler handler =
          (PropertyReadHandler) propertyHandlers.get(i);
      result.setProperty(handler.getName(), handler.getResult());
    }
  }

  /**
   * Returns the resulting properties collection, never null.
   *
   * @return the properties.
   */
  public Properties getResult()
  {
    return result;
  }

  /**
   * Returns the resulting properties collection, never null.
   *
   * @return the properties.
   * @throws SAXException if there is a parsing error.
   */
  public Object getObject() throws SAXException
  {
    return result;
  }
}
